@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -132,6 +132,15 @@ extern "C"
  */
   void eXosip_automatic_action(void);
 
+#ifndef MINISIZE
+  /**
+ * Automatic internal handling of known package.
+ * 
+ * @param evt          event related to an incoming subscription.
+ */
+  int eXosip_insubscription_automatic(eXosip_event_t *evt);
+#endif
+
 /**
  * Generate random string:
  *
@@ -163,6 +172,13 @@ extern "C"
  */
   sdp_message_t *eXosip_get_local_sdp(int did);
 
+/**
+ * Get local SDP body for the previous latest INVITE of call.
+ * 
+ * @param did          dialog id of call.
+ */
+  sdp_message_t *eXosip_get_previous_local_sdp (int did);
+
 /**
  * Get remote SDP body for the latest INVITE of call.
  * 
@@ -198,6 +214,36 @@ extern "C"
  */
   sdp_media_t *eXosip_get_audio_media(sdp_message_t *sdp);
 
+/**
+ * Get video connection information for call.
+ * 
+ * @param sdp     sdp information.
+ */
+  sdp_connection_t *eXosip_get_video_connection(sdp_message_t *sdp);
+
+/**
+ * Get video media information for call.
+ * 
+ * @param sdp     sdp information.
+ */
+  sdp_media_t *eXosip_get_video_media(sdp_message_t *sdp);
+
+/**
+ * Get media connection information for call.
+ * 
+ * @param sdp     sdp information.
+ * @param media   media to search.
+ */
+  sdp_connection_t *eXosip_get_connection (sdp_message_t * sdp, const char *media);
+
+/**
+ * Get media information for call.
+ * 
+ * @param sdp     sdp information.
+ * @param media   media to search.
+ */
+  sdp_media_t *eXosip_get_media (sdp_message_t * sdp, const char *media);
+
 /** @} */
 
 /**
