@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,73 +31,39 @@ extern eXosip_t eXosip;
 
 int ipv6_enable = 0;
 
+#ifdef OSIP_MT
 static void *_eXosip_thread (void *arg);
-static int _eXosip_execute (void);
+#endif
 static void _eXosip_keep_alive (void);
 
+#ifndef MINISIZE
+
 void
 eXosip_enable_ipv6 (int _ipv6_enable)
 {
   ipv6_enable = _ipv6_enable;
 }
 
+#endif
+
+const char *
+eXosip_get_version(void)
+{
+  return EXOSIP_VERSION;
+}
+
 void
 eXosip_masquerade_contact (const char *public_address, int port)
 {
-  if (public_address == NULL || public_address[0] == '\0')
-    {
-      memset (eXosip.net_interfaces[0].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[0].net_firewall_ip));
-      memset (eXosip.net_interfaces[1].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[1].net_firewall_ip));
-      memset (eXosip.net_interfaces[2].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[2].net_firewall_ip));
-      return;
-    }
-
-  snprintf (eXosip.net_interfaces[0].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[0].net_firewall_ip), "%s",
-            public_address);
-  snprintf (eXosip.net_interfaces[1].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[1].net_firewall_ip), "%s",
-            public_address);
-  snprintf (eXosip.net_interfaces[2].net_firewall_ip,
-            sizeof (eXosip.net_interfaces[2].net_firewall_ip), "%s",
-            public_address);
-
-  if (port > 0)
-    {
-      snprintf (eXosip.net_interfaces[0].net_port,
-                sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
-      snprintf (eXosip.net_interfaces[1].net_port,
-                sizeof (eXosip.net_interfaces[1].net_port), "%i", port);
-      snprintf (eXosip.net_interfaces[2].net_port,
-                sizeof (eXosip.net_interfaces[2].net_port), "%i", port);
-    }
+  eXtl_udp.tl_masquerade_contact(public_address, port);
+  eXtl_tcp.tl_masquerade_contact(public_address, port);
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_tls.tl_masquerade_contact(public_address, port);
+  eXtl_dtls.tl_masquerade_contact(public_address, port);
+#endif
   return;
 }
 
-int
-eXosip_force_masquerade_contact (const char *public_address)
-{
-  if (public_address == NULL || public_address[0] == '\0')
-    {
-      memset (eXosip.net_interfaces[0].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[0].net_firewall_ip));
-      memset (eXosip.net_interfaces[1].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[1].net_firewall_ip));
-      memset (eXosip.net_interfaces[2].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[2].net_firewall_ip));
-      eXosip.forced_localip = 0;
-      return 0;
-    }
-  eXosip.forced_localip = 1;
-  snprintf (eXosip.net_interfaces[0].net_firewall_ip, 50, "%s", public_address);
-  snprintf (eXosip.net_interfaces[1].net_firewall_ip, 50, "%s", public_address);
-  snprintf (eXosip.net_interfaces[2].net_firewall_ip, 50, "%s", public_address);
-  return 0;
-}
-
 int
 eXosip_guess_localip (int family, char *address, int size)
 {
@@ -163,17 +129,29 @@ eXosip_quit (void)
 {
   jauthinfo_t *jauthinfo;
   eXosip_call_t *jc;
+  eXosip_reg_t *jreg;
+#ifndef MINISIZE
   eXosip_notify_t *jn;
   eXosip_subscribe_t *js;
-  eXosip_reg_t *jreg;
   eXosip_pub_t *jpub;
+#endif
+#ifdef OSIP_MT
   int i;
-  int pos;
+#endif
+
+  if (eXosip.j_stop_ua==-1)
+  {
+		OSIP_TRACE (osip_trace
+					(__FILE__, __LINE__, OSIP_WARNING, NULL,
+					"eXosip: already stopped!\n"));
+	  return;
+  }
 
   eXosip.j_stop_ua = 1;         /* ask to quit the application */
   __eXosip_wakeup ();
   __eXosip_wakeup_event ();
 
+#ifdef OSIP_MT
   if (eXosip.j_thread!=NULL)
   {
     i = osip_thread_join ((struct osip_thread *) eXosip.j_thread);
@@ -188,6 +166,7 @@ eXosip_quit (void)
 
   jpipe_close (eXosip.j_socketctl);
   jpipe_close (eXosip.j_socketctl_event);
+#endif
 
   osip_free (eXosip.user_agent);
 
@@ -197,6 +176,7 @@ eXosip_quit (void)
       eXosip_call_free (jc);
     }
 
+#ifndef MINISIZE
   for (js = eXosip.j_subscribes; js != NULL; js = eXosip.j_subscribes)
     {
       REMOVE_ELEMENT (eXosip.j_subscribes, js);
@@ -208,35 +188,14 @@ eXosip_quit (void)
       REMOVE_ELEMENT (eXosip.j_notifies, jn);
       eXosip_notify_free (jn);
     }
+#endif
 
+#ifdef OSIP_MT
   osip_mutex_destroy ((struct osip_mutex *) eXosip.j_mutexlock);
+#if !defined (_WIN32_WCE)
   osip_cond_destroy ((struct osip_cond *) eXosip.j_cond);
-
-  if (eXosip.net_interfaces[0].net_socket)
-    {
-      close (eXosip.net_interfaces[0].net_socket);
-      eXosip.net_interfaces[0].net_socket = -1;
-    }
-  if (eXosip.net_interfaces[1].net_socket)
-    {
-      close (eXosip.net_interfaces[1].net_socket);
-      eXosip.net_interfaces[1].net_socket = -1;
-    }
-  if (eXosip.net_interfaces[2].net_socket)
-    {
-      close (eXosip.net_interfaces[2].net_socket);
-      eXosip.net_interfaces[2].net_socket = -1;
-    }
-
-  for (pos = 0; pos < EXOSIP_MAX_SOCKETS; pos++)
-    {
-      if (eXosip.net_interfaces[0].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[0].net_socket_tab[pos].socket);
-      if (eXosip.net_interfaces[1].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[1].net_socket_tab[pos].socket);
-      if (eXosip.net_interfaces[2].net_socket_tab[pos].socket != 0)
-        close (eXosip.net_interfaces[2].net_socket_tab[pos].socket);
-    }
+#endif
+#endif
 
   for (jreg = eXosip.j_reg; jreg != NULL; jreg = eXosip.j_reg)
     {
@@ -244,11 +203,13 @@ eXosip_quit (void)
       eXosip_reg_free (jreg);
     }
 
+#ifndef MINISIZE
   for (jpub = eXosip.j_pub; jpub != NULL; jpub = eXosip.j_pub)
     {
       REMOVE_ELEMENT (eXosip.j_pub, jpub);
       _eXosip_pub_free (jpub);
     }
+#endif
 
   while (!osip_list_eol (eXosip.j_transactions, 0))
     {
@@ -272,10 +233,10 @@ eXosip_quit (void)
 
   osip_free (eXosip.j_transactions);
 
-  eXosip_kill_transaction (eXosip.j_osip->osip_ict_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_nict_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_ist_transactions);
-  eXosip_kill_transaction (eXosip.j_osip->osip_nist_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_ict_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_nict_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_ist_transactions);
+  eXosip_kill_transaction (&eXosip.j_osip->osip_nist_transactions);
   osip_release (eXosip.j_osip);
 
   {
@@ -295,21 +256,40 @@ eXosip_quit (void)
       osip_free (jauthinfo);
     }
 
+  eXtl_udp.tl_free();
+  eXtl_tcp.tl_free();
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_dtls.tl_free();
+  eXtl_tls.tl_free();
+#endif
+
+  memset (&eXosip, 0, sizeof (eXosip));
+  eXosip.j_stop_ua = -1;
   return;
 }
 
 int
 eXosip_set_socket (int transport, int socket, int port)
 {
-  if (eXosip.net_interfaces[0].net_socket > 0)
-    close (eXosip.net_interfaces[0].net_socket);
-
-  eXosip.net_interfaces[0].net_protocol = transport;
-  eXosip.net_interfaces[0].net_ip_family = AF_INET;
-  eXosip.net_interfaces[0].net_socket = socket;
-  snprintf (eXosip.net_interfaces[0].net_port,
-            sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
+  eXosip.eXtl = NULL;
+  if (transport==IPPROTO_UDP)
+    {
+      eXtl_udp.proto_port = port;
+      eXtl_udp.tl_set_socket(socket);
+      eXosip.eXtl = &eXtl_udp;
+      snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "UDP");
+    }
+  else if (transport==IPPROTO_TCP)
+    {
+      eXtl_tcp.proto_port = port;
+      eXtl_tcp.tl_set_socket(socket); 
+      eXosip.eXtl = &eXtl_tcp;
+      snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TCP");
+   }
+  else
+    return -1;
 
+#ifdef OSIP_MT
   eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
   if (eXosip.j_thread == NULL)
     {
@@ -318,6 +298,7 @@ eXosip_set_socket (int transport, int socket, int port)
                    "eXosip: Cannot start thread!\n"));
       return -1;
     }
+#endif
   return 0;
 }
 
@@ -331,245 +312,302 @@ setsockopt_ipv6only (int sock)
 }
 #endif /* IPV6_V6ONLY */
 
+#ifndef MINISIZE
+int eXosip_find_free_port(int free_port, int transport)
+{
+  int res1;
+  int res2;
+  struct addrinfo *addrinfo_rtp = NULL;
+  struct addrinfo *curinfo_rtp;
+  struct addrinfo *addrinfo_rtcp = NULL;
+  struct addrinfo *curinfo_rtcp;
+  int sock;
+  int count;
+
+  for (count=0;count<8;count++)
+  {
+	  res1 = eXosip_get_addrinfo (&addrinfo_rtp, "0.0.0.0", free_port + count*2, transport);
+	  if (res1)
+		return -1;
+	  res2 = eXosip_get_addrinfo (&addrinfo_rtcp, "0.0.0.0", free_port + count*2+ 1, transport);
+	  if (res2)
+	  {
+		  eXosip_freeaddrinfo (addrinfo_rtp);
+		  return -1;
+	  }
+
+	  sock=-1;
+	  for (curinfo_rtp = addrinfo_rtp; curinfo_rtp; curinfo_rtp = curinfo_rtp->ai_next)
+		{
+		  if (curinfo_rtp->ai_protocol && curinfo_rtp->ai_protocol != transport)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+						   "eXosip: Skipping protocol %d\n", curinfo_rtp->ai_protocol));
+			  continue;
+			}
+
+		  sock = (int) socket (curinfo_rtp->ai_family, curinfo_rtp->ai_socktype,
+							   curinfo_rtp->ai_protocol);
+		  if (sock < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot create socket!\n"));
+			  continue;
+			}
+
+		  if (curinfo_rtp->ai_family == AF_INET6)
+			{
+#ifdef IPV6_V6ONLY
+			  if (setsockopt_ipv6only (sock))
+				{
+				  close (sock);
+				  sock = -1;
+				  OSIP_TRACE (osip_trace
+							  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+							   "eXosip: Cannot set socket option!\n"));
+				  continue;
+				}
+#endif /* IPV6_V6ONLY */
+			}
+
+		  res1 = bind (sock, curinfo_rtp->ai_addr, curinfo_rtp->ai_addrlen);
+		  if (res1 < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+						  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+						   curinfo_rtp->ai_family));
+			  close (sock);
+			  sock = -1;
+			  continue;
+			}
+		  break;
+	  }
+
+	  eXosip_freeaddrinfo (addrinfo_rtp);
+
+	  if (sock==-1)
+	  {
+		  eXosip_freeaddrinfo (addrinfo_rtcp);
+		  continue;
+	  }
+
+	  close(sock);
+	  sock=-1;
+	  for (curinfo_rtcp = addrinfo_rtcp; curinfo_rtcp; curinfo_rtcp = curinfo_rtcp->ai_next)
+		{
+		  if (curinfo_rtcp->ai_protocol && curinfo_rtcp->ai_protocol != transport)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+						   "eXosip: Skipping protocol %d\n", curinfo_rtcp->ai_protocol));
+			  continue;
+			}
+
+		  sock = (int) socket (curinfo_rtcp->ai_family, curinfo_rtcp->ai_socktype,
+							   curinfo_rtcp->ai_protocol);
+		  if (sock < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot create socket!\n"));
+			  continue;
+			}
+
+		  if (curinfo_rtcp->ai_family == AF_INET6)
+			{
+#ifdef IPV6_V6ONLY
+			  if (setsockopt_ipv6only (sock))
+				{
+				  close (sock);
+				  sock = -1;
+				  OSIP_TRACE (osip_trace
+							  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+							   "eXosip: Cannot set socket option!\n"));
+				  continue;
+				}
+#endif /* IPV6_V6ONLY */
+			}
+
+		  res1 = bind (sock, curinfo_rtcp->ai_addr, curinfo_rtcp->ai_addrlen);
+		  if (res1 < 0)
+			{
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+						  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+						   curinfo_rtp->ai_family));
+			  close (sock);
+			  sock = -1;
+			  continue;
+			}
+		  break;
+	  }
+
+	  eXosip_freeaddrinfo (addrinfo_rtcp);
+
+	  /* the pair must be free */
+	  if (sock==-1)
+		  continue;
+
+	  close(sock);
+	  sock=-1;
+	  return free_port + count*2;
+  }
+
+  /* just get a free port */
+  res1 = eXosip_get_addrinfo (&addrinfo_rtp, "0.0.0.0", 0, transport);
+  if (res1)
+	return -1;
+
+  sock=-1;
+  for (curinfo_rtp = addrinfo_rtp; curinfo_rtp; curinfo_rtp = curinfo_rtp->ai_next)
+	{
+	  socklen_t len;
+	  struct sockaddr_storage ai_addr;
+
+	  if (curinfo_rtp->ai_protocol && curinfo_rtp->ai_protocol != transport)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_INFO3, NULL,
+					   "eXosip: Skipping protocol %d\n", curinfo_rtp->ai_protocol));
+		  continue;
+		}
+
+	  sock = (int) socket (curinfo_rtp->ai_family, curinfo_rtp->ai_socktype,
+						   curinfo_rtp->ai_protocol);
+	  if (sock < 0)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+					   "eXosip: Cannot create socket!\n"));
+		  continue;
+		}
+
+	  if (curinfo_rtp->ai_family == AF_INET6)
+		{
+#ifdef IPV6_V6ONLY
+		  if (setsockopt_ipv6only (sock))
+			{
+			  close (sock);
+			  sock = -1;
+			  OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+						   "eXosip: Cannot set socket option!\n"));
+			  continue;
+			}
+#endif /* IPV6_V6ONLY */
+		}
+
+	  res1 = bind (sock, curinfo_rtp->ai_addr, curinfo_rtp->ai_addrlen);
+	  if (res1 < 0)
+		{
+		  OSIP_TRACE (osip_trace
+					  (__FILE__, __LINE__, OSIP_WARNING, NULL,
+					  "eXosip: Cannot bind socket node: 0.0.0.0 family:%d\n",
+					   curinfo_rtp->ai_family));
+		  close (sock);
+		  sock = -1;
+		  continue;
+		}
+
+      len = sizeof (ai_addr);
+      res1 = getsockname (sock, (struct sockaddr *) &ai_addr, &len);
+      if (res1 != 0)      
+	  {
+		  close (sock);
+		  sock = -1;
+		  continue;
+	  }
+
+	  close(sock);
+	  sock=-1;
+	  eXosip_freeaddrinfo (addrinfo_rtp);
+
+	  if (ipv6_enable == 0)
+		return ntohs (((struct sockaddr_in *) &ai_addr)->sin_port);
+	  else
+		return ntohs (((struct sockaddr_in6 *) &ai_addr)->sin6_port);
+  }
+
+  eXosip_freeaddrinfo (addrinfo_rtp);
+
+  if (sock!=-1)
+  {
+	  close(sock);
+	  sock=-1;
+  }
+
+  return -1;
+}
+#endif
+
 int
 eXosip_listen_addr (int transport, const char *addr, int port, int family,
                     int secure)
 {
-  int res;
-  struct addrinfo *addrinfo = NULL;
-  struct addrinfo *curinfo;
-  const char *node = addr;
-  int sock = -1;
-  struct eXosip_net *net_int;
-  char localip[256];
+  int i=-1;
+  struct eXtl_protocol *eXtl=NULL;
 
-  if (transport == IPPROTO_UDP)
-    net_int = &eXosip.net_interfaces[0];
-  else if (transport == IPPROTO_TCP)
-    net_int = &eXosip.net_interfaces[1];
-  else if (transport == IPPROTO_TCP && secure != 0)
-    net_int = &eXosip.net_interfaces[2];
-  else
+  if (eXosip.eXtl!=NULL)
     {
+      /* already set */
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unknown protocol (use IPPROTO_UDP or IPPROTO_TCP!\n"));
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: already listening somewhere\n"));
       return -1;
     }
 
-  if (eXosip.http_port)
-    {
-      /* USE TUNNEL CAPABILITY */
-      transport = IPPROTO_TCP;
-    }
+  if (transport==IPPROTO_UDP && secure==0)
+    eXtl = &eXtl_udp;
+  else if (transport==IPPROTO_TCP && secure==0)
+    eXtl = &eXtl_tcp;
+#ifdef HAVE_OPENSSL_SSL_H
+  else if (transport==IPPROTO_UDP)
+    eXtl = &eXtl_dtls;
+  else if (transport==IPPROTO_TCP)
+    eXtl = &eXtl_tls;
+#endif
 
-  if (port < 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: port must be higher than 0!\n"));
-      return -1;
-    }
-
-  net_int->net_ip_family = family;
-  if (family == AF_INET6)
-    {
-      ipv6_enable = 1;
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "IPv6 is enabled. Pls report bugs\n"));
-    }
-
-  eXosip_guess_localip (net_int->net_ip_family, localip, sizeof (localip));
-  if (localip[0] == '\0')
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: No ethernet interface found!\n"));
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: using 127.0.0.1 (debug mode)!\n"));
-      /* we should always fallback on something. The linphone user will surely
-         start linphone BEFORE setting its dial up connection. */
-    }
-
-  if (!node)
-    {
-      node = ipv6_enable ? "::" : "0.0.0.0";
-    }
-
-
-  res = eXosip_get_addrinfo (&addrinfo, node, port, transport);
-  if (res)
+  if (eXtl==NULL)
     return -1;
 
-  for (curinfo = addrinfo; curinfo; curinfo = curinfo->ai_next)
+  eXtl->proto_family=family;
+  eXtl->proto_port=port;
+  if (addr!=NULL)
+    snprintf(eXtl->proto_ifs, sizeof(eXtl->proto_ifs), "%s", addr);
+
+  i = eXtl->tl_open();
+  
+  if (i!=0)
+    return -1;
+  
+  eXosip.eXtl = eXtl;
+
+  if (transport==IPPROTO_UDP && secure==0)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "UDP");
+  else if (transport==IPPROTO_TCP && secure==0)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TCP");
+  else if (transport==IPPROTO_UDP)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "DTLS-UDP");
+  else if (transport==IPPROTO_TCP)
+    snprintf(eXosip.transport, sizeof(eXosip.transport), "%s", "TLS");
+
+#ifdef OSIP_MT
+  if (eXosip.j_thread==NULL)
     {
-      socklen_t len;
-
-      if (curinfo->ai_protocol && curinfo->ai_protocol != transport)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO3, NULL,
-                       "eXosip: Skipping protocol %d\n", curinfo->ai_protocol));
-          continue;
-        }
-
-      sock = (int) socket (curinfo->ai_family, curinfo->ai_socktype,
-                           curinfo->ai_protocol);
-      if (sock < 0)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot create socket!\n", strerror (errno)));
-          continue;
-        }
-
-      if (eXosip.http_port)
-        {
-          break;
-        }
-
-      if (curinfo->ai_family == AF_INET6)
-        {
-#ifdef IPV6_V6ONLY
-          if (setsockopt_ipv6only (sock))
-            {
-              close (sock);
-              sock = -1;
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot set socket option!\n",
-                           strerror (errno)));
-              continue;
-            }
-#endif /* IPV6_V6ONLY */
-        }
-
-      res = bind (sock, curinfo->ai_addr, curinfo->ai_addrlen);
-      if (res < 0)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot bind socket node:%s family:%d %s\n",
-                       node, curinfo->ai_family, strerror (errno)));
-          close (sock);
-          sock = -1;
-          continue;
-        }
-      len = sizeof (net_int->ai_addr);
-      res = getsockname (sock, (struct sockaddr *) &net_int->ai_addr, &len);
-      if (res != 0)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot get socket name (%s)\n", strerror (errno)));
-          memcpy (&net_int->ai_addr, curinfo->ai_addr, curinfo->ai_addrlen);
-        }
-
-      if (transport != IPPROTO_UDP)
-        {
-          res = listen (sock, SOMAXCONN);
-          if (res < 0)
-            {
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot bind socket node:%s family:%d %s\n",
-                           node, curinfo->ai_family, strerror (errno)));
-              close (sock);
-              sock = -1;
-              continue;
-            }
-        }
-
-      break;
-    }
-
-  freeaddrinfo (addrinfo);
-
-  if (sock < 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: Cannot bind on port: %i\n", port));
-      return -1;
-    }
-
-  if (eXosip.http_port)
-    net_int->net_protocol = IPPROTO_UDP;
-  else
-    net_int->net_protocol = transport;
-  net_int->net_socket = sock;
-
-  if (port == 0)
-    {
-      /* get port number from socket */
-      if (ipv6_enable == 0)
-        port = ntohs (((struct sockaddr_in *) &net_int->ai_addr)->sin_port);
-      else
-        port = ntohs (((struct sockaddr_in6 *) &net_int->ai_addr)->sin6_port);
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "eXosip: Binding on port %i!\n", port));
-    }
-
-  snprintf (net_int->net_port, sizeof (net_int->net_port) - 1, "%i", port);
-
-
-
-  if (eXosip.http_port)
-    {
-      /* only ipv4 */
-      struct sockaddr_in _addr;
-      char http_req[2048];
-      char http_reply[2048];
-      int len;
-
-      _addr.sin_port = (unsigned short) htons (eXosip.http_port);
-      _addr.sin_addr.s_addr = inet_addr (eXosip.http_proxy);
-      _addr.sin_family = PF_INET;
-
-      if (connect
-          (net_int->net_socket, (struct sockaddr *) &_addr, sizeof (_addr)) == -1)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Failed to connect to http server on %s:%i!\n",
-                       eXosip.http_proxy, port));
-          return -1;
-        }
-
-      sprintf (http_req, "GET / HTTP/1.1\r\nUdpHost: %s:%d\r\n\r\n",
-               eXosip.http_outbound_proxy, 5060);
-
-      len = send (net_int->net_socket, http_req, (int) strlen (http_req), 0);
-
-      if (len < 0)
-        return -1;
-
-      osip_usleep (50000);
-
-      if ((len =
-           recv (net_int->net_socket, http_reply, sizeof (http_reply), 0)) > 0)
-        http_reply[len] = '\0';
-      else
-        return -1;
-
-      if (strncmp (http_reply, "HTTP/1.0 200 OK\r\n", 17) == 0
-          || strncmp (http_reply, "HTTP/1.1 200 OK\r\n", 17) == 0)
-        {
-      } else
-        return -1;
-    }
-
-  eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
-  if (eXosip.j_thread == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+      eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
+      if (eXosip.j_thread == NULL)
+	{
+	  OSIP_TRACE (osip_trace
+		      (__FILE__, __LINE__, OSIP_ERROR, NULL,
                    "eXosip: Cannot start thread!\n"));
-      return -1;
+	  return -1;
+	}
     }
+#endif
+
   return 0;
 }
 
@@ -580,6 +618,12 @@ eXosip_init (void)
 
   memset (&eXosip, 0, sizeof (eXosip));
 
+  snprintf(eXosip.ipv4_for_gateway, 256, "%s", "217.12.3.11");
+  snprintf(eXosip.ipv6_for_gateway, 256, "%s", "2001:638:500:101:2e0:81ff:fe24:37c6");
+#ifndef MINISIZE
+  snprintf(eXosip.event_package, 256, "%s", "dialog");
+#endif
+
 #ifdef WIN32
   /* Initializing windows socket library */
   {
@@ -603,14 +647,20 @@ eXosip_init (void)
 
   eXosip.j_calls = NULL;
   eXosip.j_stop_ua = 0;
+#ifdef OSIP_MT
   eXosip.j_thread = NULL;
+#endif
   eXosip.j_transactions = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
   osip_list_init (eXosip.j_transactions);
   eXosip.j_reg = NULL;
 
+#ifdef OSIP_MT
+#if !defined (_WIN32_WCE)
   eXosip.j_cond = (struct osip_cond *) osip_cond_init ();
+#endif
 
   eXosip.j_mutexlock = (struct osip_mutex *) osip_mutex_init ();
+#endif
 
   if (-1 == osip_init (&osip))
     {
@@ -626,29 +676,42 @@ eXosip_init (void)
 
   eXosip.j_osip = osip;
 
+#ifdef OSIP_MT
   /* open a TCP socket to wake up the application when needed. */
   eXosip.j_socketctl = jpipe ();
   if (eXosip.j_socketctl == NULL)
-    return -1;
+    return -2;
 
   eXosip.j_socketctl_event = jpipe ();
   if (eXosip.j_socketctl_event == NULL)
-    return -1;
+    return -3;
+#endif
 
   /* To be changed in osip! */
   eXosip.j_events = (osip_fifo_t *) osip_malloc (sizeof (osip_fifo_t));
   osip_fifo_init (eXosip.j_events);
 
+  eXosip.use_rport = 1;
+
+  eXosip.keep_alive = 32000;
+
+  eXtl_udp.tl_init();
+  eXtl_tcp.tl_init();
+#ifdef HAVE_OPENSSL_SSL_H
+  eXtl_dtls.tl_init();
+  eXtl_tls.tl_init();
+#endif
   return 0;
 }
 
 
-static int
-_eXosip_execute (void)
+int
+eXosip_execute (void)
 {
   struct timeval lower_tv;
   int i;
 
+#ifdef OSIP_MT
   osip_timers_gettimeout (eXosip.j_osip, &lower_tv);
   if (lower_tv.tv_sec > 15)
     {
@@ -672,6 +735,10 @@ _eXosip_execute (void)
                    "eXosip: timer sec:%i usec:%i!\n",
                    lower_tv.tv_sec, lower_tv.tv_usec));
     }
+#else
+  lower_tv.tv_sec = 0;
+  lower_tv.tv_usec = 0;
+#endif
   i = eXosip_read_message (1, lower_tv.tv_sec, lower_tv.tv_usec);
 
   if (i == -2)
@@ -693,6 +760,9 @@ _eXosip_execute (void)
   /* free all Calls that are in the TERMINATED STATE? */
   eXosip_release_terminated_calls ();
   eXosip_release_terminated_registrations ();
+#ifndef MINISIZE
+  eXosip_release_unused_transactions();
+#endif
 
   eXosip_unlock ();
 
@@ -713,14 +783,125 @@ eXosip_set_option (eXosip_option opt, const void *value)
 
   switch (opt)
     {
+      case EXOSIP_OPT_ADD_ACCOUNT_INFO:
+		  {
+			  struct eXosip_account_info *ainfo;
+			  int i;
+			  ainfo = (struct eXosip_account_info*) value;
+			  if (ainfo==NULL || ainfo->proxy[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  for (i=0;i<MAX_EXOSIP_ACCOUNT_INFO;i++)
+			  {
+				  if (eXosip.account_entries[i].proxy[0]!='\0'
+					  &&0==osip_strcasecmp(eXosip.account_entries[i].proxy, ainfo->proxy))
+				  {
+					  /* update ainfo */
+					  if (ainfo->nat_ip[0]!='\0')
+					  {
+						  snprintf(eXosip.account_entries[i].nat_ip, sizeof(eXosip.account_entries[i].nat_ip), "%s", ainfo->nat_ip);
+						  eXosip.account_entries[i].nat_port = ainfo->nat_port;
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO1, NULL,
+										"eXosip option set: account info updated:%s -> %s:%i\n",
+										ainfo->proxy, ainfo->nat_ip, ainfo->nat_port));
+					  }
+					  else
+					  {
+						  eXosip.account_entries[i].proxy[0]='\0';
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO2, NULL,
+										 "eXosip option set: account info deleted :%s\n", ainfo->proxy));
+					  }
+					  return 0;
+				  }
+			  }
+			  if (ainfo->nat_ip[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  /* not found case: */
+			  for (i=0;i<MAX_EXOSIP_ACCOUNT_INFO;i++)
+			  {
+				  if (eXosip.account_entries[i].proxy[0]=='\0')
+				  {
+					  /* add ainfo */
+					  snprintf(eXosip.account_entries[i].proxy, sizeof(ainfo->proxy), "%s", ainfo->proxy);
+					  snprintf(eXosip.account_entries[i].nat_ip, sizeof(ainfo->nat_ip), "%s", ainfo->nat_ip);
+					  eXosip.account_entries[i].nat_port = ainfo->nat_port;
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO1, NULL,
+										"eXosip option set: account info added:%s -> %s:%i\n",
+										ainfo->proxy, ainfo->nat_ip, ainfo->nat_port));
+					  return 0;
+				  }
+			  }
+			  return -1;
+		  }
+        break;
+	  case EXOSIP_OPT_ADD_DNS_CACHE:
+		  {
+			  struct eXosip_dns_cache *entry;
+			  int i;
+			  entry = (struct eXosip_dns_cache*) value;
+			  if (entry==NULL || entry->host[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  for (i=0;i<MAX_EXOSIP_DNS_ENTRY;i++)
+			  {
+				  if (eXosip.dns_entries[i].host[0]!='\0'
+					  &&0==osip_strcasecmp(eXosip.dns_entries[i].host, entry->host))
+				  {
+					  /* update entry */
+					  if (entry->ip[0]!='\0')
+					  {
+						  snprintf(eXosip.dns_entries[i].ip, sizeof(eXosip.dns_entries[i].ip), "%s", entry->ip);
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO1, NULL,
+										 "eXosip option set: dns cache updated:%s -> %s\n", entry->host, entry->ip));
+					  }
+					  else
+					  {
+						  eXosip.dns_entries[i].host[0]='\0';
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO2, NULL,
+										 "eXosip option set: dns cache deleted :%s\n", entry->host));
+					  }
+					  return 0;
+				  }
+			  }
+			  if (entry->ip[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  /* not found case: */
+			  for (i=0;i<MAX_EXOSIP_DNS_ENTRY;i++)
+			  {
+				  if (eXosip.dns_entries[i].host[0]=='\0')
+				  {
+					  /* add entry */
+					  snprintf(eXosip.dns_entries[i].host, sizeof(entry->host), "%s", entry->host);
+					  snprintf(eXosip.dns_entries[i].ip, sizeof(entry->ip), "%s", entry->ip);
+					  OSIP_TRACE (osip_trace
+									(__FILE__, __LINE__, OSIP_INFO2, NULL,
+									 "eXosip option set: dns cache added:%s -> %s\n", entry->host, entry->ip));
+					  return 0;
+				  }
+			  }
+			  return -1;
+		  }
+        break;
       case EXOSIP_OPT_UDP_KEEP_ALIVE:
         val = *((int *) value);
         eXosip.keep_alive = val;        /* value in ms */
         break;
       case EXOSIP_OPT_UDP_LEARN_PORT:
         val = *((int *) value);
-        eXosip.learn_port = val;        /* value in ms */
+        eXosip.learn_port = val;        /* 1 to learn port */
         break;
+#ifndef MINISIZE
       case EXOSIP_OPT_SET_HTTP_TUNNEL_PORT:
         val = *((int *) value);
         eXosip.http_port = val; /* value in ms */
@@ -749,6 +930,55 @@ eXosip_set_option (eXosip_option opt, const void *value)
                      eXosip.http_outbound_proxy));
         break;
 
+      case EXOSIP_OPT_DONT_SEND_101:
+        val = *((int *) value);
+        eXosip.dontsend_101 = val;        /* 0 to disable */
+        break;
+#endif
+
+      case EXOSIP_OPT_USE_RPORT:
+        val = *((int *) value);
+        eXosip.use_rport = val;       /* 0 to disable (for broken NAT only?) */
+        break;
+
+      case EXOSIP_OPT_SET_IPV4_FOR_GATEWAY:
+        tmp = (char *) value;
+        memset (eXosip.ipv4_for_gateway, '\0',
+                sizeof (eXosip.ipv4_for_gateway));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.ipv4_for_gateway, tmp, sizeof (eXosip.ipv4_for_gateway));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: ipv4_for_gateway:%s!\n",
+                     eXosip.ipv4_for_gateway));
+        break;
+#ifndef MINISIZE
+      case EXOSIP_OPT_SET_IPV6_FOR_GATEWAY:
+        tmp = (char *) value;
+        memset (eXosip.ipv6_for_gateway, '\0',
+                sizeof (eXosip.ipv6_for_gateway));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.ipv6_for_gateway, tmp, sizeof (eXosip.ipv6_for_gateway));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: ipv6_for_gateway:%s!\n",
+                     eXosip.ipv6_for_gateway));
+        break;
+      case EXOSIP_OPT_EVENT_PACKAGE:
+        tmp = (char *) value;
+        memset (eXosip.event_package, '\0',
+                sizeof (eXosip.event_package));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.event_package, tmp, sizeof (eXosip.event_package));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: event_package:%s!\n",
+                     eXosip.event_package));
+        break;
+#endif
+
+    default:
+      return -1;
     }
   return 0;
 }
@@ -758,9 +988,6 @@ _eXosip_keep_alive (void)
 {
   static struct timeval mtimer = { 0, 0 };
 
-  eXosip_reg_t *jr;
-  struct eXosip_net *net;
-  char buf[4] = "jaK";
   struct timeval now;
 
   osip_gettimeofday (&now, NULL);
@@ -781,27 +1008,10 @@ _eXosip_keep_alive (void)
   osip_gettimeofday (&mtimer, NULL);
   add_gettimeofday (&mtimer, eXosip.keep_alive);
 
-  net = &eXosip.net_interfaces[0];
-  if (net == NULL)
-    {
-      return;
-    }
-
-  for (jr = eXosip.j_reg; jr != NULL; jr = jr->next)
-    {
-      if (jr->len > 0)
-        {
-          if (sendto (net->net_socket, (const void *) buf, 4, 0,
-                      (struct sockaddr *) &(jr->addr), jr->len) > 0)
-            {
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                           "eXosip: Keep Alive sent on UDP!\n"));
-            }
-        }
-    }
+  eXtl_udp.tl_keepalive();
 }
 
+#ifdef OSIP_MT
 void *
 _eXosip_thread (void *arg)
 {
@@ -809,10 +1019,12 @@ _eXosip_thread (void *arg)
 
   while (eXosip.j_stop_ua == 0)
     {
-      i = _eXosip_execute ();
+      i = eXosip_execute ();
       if (i == -2)
         osip_thread_exit ();
     }
   osip_thread_exit ();
   return NULL;
 }
+
+#endif
