@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@
 
 
 #ifndef  WIN32
+#if !defined(__arc__)
 #include <memory.h>
+#endif
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -41,6 +43,8 @@ static jauthinfo_t *eXosip_find_authentication_info (const char *username,
 
 eXosip_t eXosip;
 
+#ifdef OSIP_MT
+
 void
 __eXosip_wakeup (void)
 {
@@ -53,7 +57,6 @@ __eXosip_wakeup_event (void)
   jpipe_write (eXosip.j_socketctl_event, "w", 1);
 }
 
-
 int
 eXosip_lock (void)
 {
@@ -66,6 +69,34 @@ eXosip_unlock (void)
   return osip_mutex_unlock ((struct osip_mutex *) eXosip.j_mutexlock);
 }
 
+#endif
+
+int
+_eXosip_transaction_init (osip_transaction_t ** transaction,
+                       osip_fsm_type_t ctx_type, osip_t * osip,
+                       osip_message_t * message)
+{
+#ifdef SRV_RECORD
+	osip_srv_record_t record;
+#endif
+	int i;
+    i = osip_transaction_init (transaction, ctx_type, osip, message);
+	if (i != 0)
+	{
+		return i;
+	}
+#ifdef SRV_RECORD
+	memset(&record, 0, sizeof(osip_srv_record_t));
+	i = _eXosip_srv_lookup(*transaction, message, &record);
+	if (i<0)
+	{
+		return 0;
+	}
+	osip_transaction_set_srv_record(*transaction, &record);
+#endif
+	return 0;
+}
+
 int
 eXosip_transaction_find (int tid, osip_transaction_t ** transaction)
 {
@@ -87,6 +118,8 @@ eXosip_transaction_find (int tid, osip_transaction_t ** transaction)
   return -1;
 }
 
+#ifndef MINISIZE
+
 static int
 _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
                          int *retry)
@@ -97,9 +130,7 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   osip_event_t *sipevent;
   jinfo_t *ji = NULL;
 
-  char locip[256];
   int cseq;
-  char tmp[256];
   osip_via_t *via;
   int i;
 
@@ -130,7 +161,7 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       return -1;
     }
 
-  via = (osip_via_t *) osip_list_get (msg->vias, 0);
+  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
   if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
     {
       osip_message_free (msg);
@@ -149,36 +180,9 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       jd->d_dialog->local_cseq++;
     }
 
-  osip_list_remove (msg->vias, 0);
-  osip_via_free (via);
-  i = _eXosip_find_protocol (out_tr->orig_request);
-  if (i == IPPROTO_UDP)
+  i = eXosip_update_top_via(msg);
+  if (i!=0)
     {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip,
-                               sizeof (locip));
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-  } else
-    {
-      /* tls? */
       osip_message_free (msg);
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -186,10 +190,6 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
       return -1;
     }
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (msg->vias, via, 0);
-
   if (eXosip_add_authentication_information (msg, out_tr->last_response) < 0)
     {
       osip_message_free (msg);
@@ -199,9 +199,9 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   osip_message_force_update (msg);
 
   if (MSG_IS_INVITE (msg))
-    i = osip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
+    i = _eXosip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
   else
-    i = osip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
+    i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
 
   if (i != 0)
     {
@@ -236,7 +236,6 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   return 0;
 }
 
-
 static int
 _eXosip_retry_register_with_auth (eXosip_event_t * je)
 {
@@ -285,7 +284,7 @@ _eXosip_retry_invite_with_auth (eXosip_event_t * je)
   if (*retry < 3)
     {
       (*retry)++;
-      return _eXosip_call_send_request_with_credential(jc, jd, tr);
+      return _eXosip_call_retry_request(jc, jd, tr);
     }
   return -1;
 }
@@ -307,7 +306,7 @@ _eXosip_redirect_invite (eXosip_event_t * je)
       return -1;
     }
   
-  return _eXosip_call_redirect_request (jc, jd, tr);
+  return _eXosip_call_retry_request (jc, jd, tr);
 }
 
 static int
@@ -389,6 +388,7 @@ eXosip_retry_with_auth (eXosip_event_t * je)
     {
       return _eXosip_retry_invite_with_auth (je);
     }
+#ifndef MINISIZE
   else if (je->sid>0)
     {
       return _eXosip_retry_subscribe_with_auth (je);
@@ -399,6 +399,7 @@ eXosip_retry_with_auth (eXosip_event_t * je)
     }
   else if (MSG_IS_PUBLISH (je->request))
     return _eXosip_retry_publish_with_auth (je);
+#endif
   else
     {
       osip_transaction_t *tr=NULL;
@@ -439,7 +440,6 @@ _eXosip_redirect (eXosip_event_t * je)
     }
 }
 
-
 int
 eXosip_default_action (eXosip_event_t * je)
 {
@@ -454,7 +454,6 @@ eXosip_default_action (eXosip_event_t * je)
     return 1;
 }
 
-
 void
 eXosip_automatic_refresh (void)
 {
@@ -484,14 +483,12 @@ eXosip_automatic_refresh (void)
                 {               /* will expires in 60 sec: send refresh! */
                   int i;
 
-                  i =
-                    _eXosip_subscribe_send_request_with_credential (js,
-                                                                    jd, out_tr);
-                  if (i != 0)
+				  i = _eXosip_subscribe_automatic_refresh(js, jd, out_tr);
+				  if (i != 0)
                     {
                       OSIP_TRACE (osip_trace
                                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                                   "eXosip: could not clone subscribe for refresh\n"));
+                                   "eXosip: could not send subscribe for refresh\n"));
                     }
                 }
             }
@@ -523,14 +520,61 @@ eXosip_automatic_refresh (void)
         }
     }
 }
+#endif
+
+void
+eXosip_retransmit_lost200ok()
+{
+	eXosip_call_t *jc;
+	eXosip_dialog_t *jd;
+	time_t now;
+
+	now = time (NULL);
+
+	for (jc = eXosip.j_calls; jc != NULL; jc = jc->next)
+	{
+		if (jc->c_id >= 1 && jc->c_dialogs != NULL)
+		{
+			for (jd = jc->c_dialogs; jd != NULL; jd = jd->next)
+			{
+				if (jd->d_id >=1 && jd->d_dialog != NULL && jd->d_200Ok!=NULL)
+				{
+					if (jd->d_count==5)
+					{
+						OSIP_TRACE (osip_trace
+									(__FILE__, __LINE__, OSIP_ERROR, NULL,
+									"eXosip: no ACK received during 20s: dropping call\n"));
+						/* hard for users to detect than I sent this BYE... */
+						jd->d_count=0;
+						osip_message_free(jd->d_200Ok);
+						jd->d_200Ok=NULL;
+						eXosip_call_terminate(jc->c_id, jd->d_id);
+					}
+					else if (jd->d_timer<now)
+					{
+						/* a dialog exist: retransmit lost 200ok */
+						jd->d_timer = time (NULL) + 4;
+						jd->d_count++;
+						jd = jc->c_dialogs;
+						/* TU retransmission */
+						cb_snd_message (NULL, jd->d_200Ok, NULL,0, -1);
+					}
+				}
+			}
+		}
+	}
+	return;
+}
 
 void
 eXosip_automatic_action (void)
 {
   eXosip_call_t *jc;
-  eXosip_subscribe_t *js;
   eXosip_dialog_t *jd;
+#ifndef MINISIZE
+  eXosip_subscribe_t *js;
   eXosip_notify_t *jn;
+#endif
 
   eXosip_reg_t *jr;
   time_t now;
@@ -565,7 +609,7 @@ eXosip_automatic_action (void)
                 {
                   int i;
 
-                  i = _eXosip_call_send_request_with_credential (jc, NULL, out_tr);
+                  i = _eXosip_call_retry_request (jc, NULL, out_tr);
                   if (i != 0)
                     {
                       OSIP_TRACE (osip_trace
@@ -588,7 +632,7 @@ eXosip_automatic_action (void)
               /* retry with credential */
               int i;
 
-              i = _eXosip_call_redirect_request (jc, NULL, out_tr);
+              i = _eXosip_call_retry_request (jc, NULL, out_tr);
               if (i != 0)
                 {
                   OSIP_TRACE (osip_trace
@@ -627,7 +671,7 @@ eXosip_automatic_action (void)
                       int i;
 
                       i =
-                        _eXosip_call_send_request_with_credential (jc, jd, out_tr);
+                        _eXosip_call_retry_request (jc, jd, out_tr);
                       if (i != 0)
                         {
                           OSIP_TRACE (osip_trace
@@ -650,7 +694,7 @@ eXosip_automatic_action (void)
                   /* retry with credential */
                   int i;
 
-                  i = _eXosip_call_redirect_request (jc, jd, out_tr);
+                  i = _eXosip_call_retry_request (jc, jd, out_tr);
                   if (i != 0)
                     {
                       OSIP_TRACE (osip_trace
@@ -662,6 +706,8 @@ eXosip_automatic_action (void)
         }
     }
 
+#ifndef MINISIZE
+
   for (js = eXosip.j_subscribes; js != NULL; js = js->next)
     {
       if (js->s_id < 1)
@@ -725,10 +771,9 @@ eXosip_automatic_action (void)
                       if (jd->d_retry < 3)
                         {
                           int i;
-
-                          i =
-                            _eXosip_subscribe_send_request_with_credential
-                            (js, jd, out_tr);
+						  i =
+							_eXosip_subscribe_send_request_with_credential (js, jd,
+																			out_tr);
                           if (i != 0)
                             {
                               OSIP_TRACE (osip_trace
@@ -744,10 +789,7 @@ eXosip_automatic_action (void)
                     {           /* will expires in 60 sec: send refresh! */
                       int i;
 
-                      i =
-                        _eXosip_subscribe_send_request_with_credential (js,
-                                                                        jd,
-                                                                        out_tr);
+					  i = _eXosip_subscribe_automatic_refresh(js, jd, out_tr);
                       if (i != 0)
                         {
                           OSIP_TRACE (osip_trace
@@ -805,6 +847,7 @@ eXosip_automatic_action (void)
         }
     }
 
+#endif
 
   for (jr = eXosip.j_reg; jr != NULL; jr = jr->next)
     {
@@ -849,8 +892,10 @@ eXosip_update ()
 {
   static int static_id = 1;
   eXosip_call_t *jc;
+#ifndef MINISIZE
   eXosip_subscribe_t *js;
   eXosip_notify_t *jn;
+#endif
   eXosip_dialog_t *jd;
   time_t now;
 
@@ -879,6 +924,7 @@ eXosip_update ()
         }
     }
 
+#ifndef MINISIZE
   for (js = eXosip.j_subscribes; js != NULL; js = js->next)
     {
       if (js->s_id < 1)
@@ -920,6 +966,7 @@ eXosip_update ()
             jd->d_id = -1;
         }
     }
+#endif
 }
 
 static jauthinfo_t *
@@ -954,7 +1001,7 @@ eXosip_find_authentication_info (const char *username, const char *realm)
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_INFO3, NULL,
                    "INFO: authinfo: %s %s\n", realm, authinfo->realm));
-      if (authinfo->realm == NULL || authinfo->realm[0] == '\0')
+      if ((authinfo->realm == NULL || authinfo->realm[0] == '\0') && fallback==NULL)
         {
           fallback = authinfo;
       } else if (strcmp (realm, authinfo->realm) == 0
@@ -1041,6 +1088,72 @@ eXosip_add_authentication_information (osip_message_t * req,
       return -1;
    }
 
+  if (last_response==NULL)
+  {
+	  /* we can add all credential that belongs to the same call-id */
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, req->call_id->number)==0)
+		{
+			char *uri;
+
+			authinfo = eXosip_find_authentication_info (req->from->url->username,
+												  http_auth->wa->realm);
+			if (authinfo == NULL)
+			{
+				OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO2, NULL,
+						   "authinfo: No authentication found for %s %s\n",
+						   req->from->url->username, http_auth->wa->realm));
+				return -1;
+			}
+
+			i = osip_uri_to_str (req->req_uri, &uri);
+			if (i != 0)
+				return -1;
+
+			http_auth->iNonceCount++;
+			if (osip_strcasecmp(req->sip_method, "REGISTER")==0)
+				i = __eXosip_create_authorization_header (http_auth->wa, uri,
+													authinfo->userid,
+													authinfo->passwd,
+													authinfo->ha1, &aut,
+													req->sip_method,
+													http_auth->pszCNonce,
+													http_auth->iNonceCount);
+			else
+				i = __eXosip_create_proxy_authorization_header (http_auth->wa, uri,
+													authinfo->userid,
+													authinfo->passwd,
+													authinfo->ha1, &aut,
+													req->sip_method,
+													http_auth->pszCNonce,
+													http_auth->iNonceCount);
+
+			osip_free (uri);
+			if (i != 0)
+				return -1;
+
+			if (aut != NULL)
+			{
+				if (osip_strcasecmp(req->sip_method, "REGISTER")==0)
+					osip_list_add (&req->authorizations, aut, -1);
+				else
+					osip_list_add (&req->proxy_authorizations, aut, -1);
+				osip_message_force_update (req);
+			}
+		}
+	}
+	return 0;
+  }
+
   pos = 0;
   osip_message_get_www_authenticate (last_response, pos, &wwwauth);
   osip_message_get_proxy_authenticate (last_response, pos, &proxyauth);
@@ -1071,21 +1184,41 @@ eXosip_add_authentication_information (osip_message_t * req,
       if (i != 0)
         return -1;
 
-      i = __eXosip_create_authorization_header (last_response, uri,
+      i = __eXosip_create_authorization_header (wwwauth, uri,
                                                 authinfo->userid,
                                                 authinfo->passwd,
                                                 authinfo->ha1, &aut,
-                                                req->sip_method);
+                                                req->sip_method,
+												"0a4f113b",
+												1);
       osip_free (uri);
       if (i != 0)
         return -1;
 
       if (aut != NULL)
         {
-          osip_list_add (req->authorizations, aut, -1);
+          osip_list_add (&req->authorizations, aut, -1);
           osip_message_force_update (req);
         }
 
+	  if (wwwauth->qop_options!=NULL)
+	  {
+		  if (osip_strcasecmp(req->sip_method, "REGISTER")==0
+			  || osip_strcasecmp(req->sip_method, "INVITE")==0
+  			  || osip_strcasecmp(req->sip_method, "SUBSCRIBE")==0)
+		  	  _eXosip_store_nonce(req->call_id->number, wwwauth);
+		  else
+		  {
+			  osip_generic_param_t *to_tag=NULL;
+			  osip_from_param_get_byname (req->to, "tag", &to_tag);
+			  if (to_tag!=NULL)
+			  {
+				  /* if message is part of a dialog */
+			  	  _eXosip_store_nonce(req->call_id->number, wwwauth);
+			  }
+		  }
+	  }
+
       pos++;
       osip_message_get_www_authenticate (last_response, pos, &wwwauth);
     }
@@ -1112,21 +1245,41 @@ eXosip_add_authentication_information (osip_message_t * req,
       if (i != 0)
         return -1;
 
-      i = __eXosip_create_proxy_authorization_header (last_response, uri,
+      i = __eXosip_create_proxy_authorization_header (proxyauth, uri,
                                                       authinfo->userid,
                                                       authinfo->passwd,
                                                       authinfo->ha1,
-                                                      &proxy_aut, req->sip_method);
+                                                      &proxy_aut, req->sip_method,
+													  "0a4f113b",
+													  1);
       osip_free (uri);
       if (i != 0)
         return -1;
 
       if (proxy_aut != NULL)
         {
-          osip_list_add (req->proxy_authorizations, proxy_aut, -1);
+          osip_list_add (&req->proxy_authorizations, proxy_aut, -1);
           osip_message_force_update (req);
         }
 
+	  if (proxyauth->qop_options!=NULL)
+	  {
+		  if (osip_strcasecmp(req->sip_method, "REGISTER")==0
+			  || osip_strcasecmp(req->sip_method, "INVITE")==0
+  			  || osip_strcasecmp(req->sip_method, "SUBSCRIBE")==0)
+		  	  _eXosip_store_nonce(req->call_id->number, proxyauth);
+		  else
+		  {
+			  osip_generic_param_t *to_tag=NULL;
+			  osip_from_param_get_byname (req->to, "tag", &to_tag);
+			  if (to_tag!=NULL)
+			  {
+				  /* if message is part of a dialog */
+			  	  _eXosip_store_nonce(req->call_id->number, proxyauth);
+			  }
+		  }
+	  }
+
       pos++;
       osip_message_get_proxy_authenticate (last_response, pos, &proxyauth);
     }
@@ -1137,56 +1290,33 @@ eXosip_add_authentication_information (osip_message_t * req,
 int
 eXosip_update_top_via (osip_message_t * sip)
 {
-  char locip[50];
-  char *tmp = (char *) osip_malloc (256 * sizeof (char));
-  osip_via_t *via = (osip_via_t *) osip_list_get (sip->vias, 0);
-  int i;
+  unsigned int number;
+  char tmp[40];
+  osip_generic_param_t *br=NULL;
+  osip_via_t *via = (osip_via_t *) osip_list_get (&sip->vias, 0);
 
-  i = _eXosip_find_protocol (sip);
-
-  osip_list_remove (sip->vias, 0);
-  osip_via_free (via);
-#ifdef SM
-  eXosip_get_localip_for (sip->req_uri->host, locip, 49);
-#else
-  if (i == IPPROTO_UDP)
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-  else if (i == IPPROTO_TCP)
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip, 49);
-  else
+  if (via==NULL)
     {
       OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (using default UDP)\n"));
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "missing via in SIP message\n"));
+      return -1;
     }
-#endif
-  if (i == IPPROTO_UDP)
+  /* browse parameter and replace "branch" */
+  osip_via_param_get_byname (via, "branch", &br);
+
+  if (br==NULL || br->gvalue==NULL)
     {
-      if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/UDP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/UDP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[0].net_port,
-                  via_branch_new_random ());
-  } else if (i == IPPROTO_TCP)
-    {
-      if (eXosip.net_interfaces[1].net_ip_family == AF_INET6)
-        snprintf (tmp, 256, "SIP/2.0/TCP [%s]:%s;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
-      else
-        snprintf (tmp, 256, "SIP/2.0/TCP %s:%s;rport;branch=z9hG4bK%u",
-                  locip, eXosip.net_interfaces[1].net_port,
-                  via_branch_new_random ());
+      OSIP_TRACE (osip_trace
+		  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		   "missing branch parameter via in SIP message\n"));
+      return -1;
     }
+  
+  osip_free(br->gvalue);
+  number = osip_build_random_number ();
 
-  osip_via_init (&via);
-  osip_via_parse (via, tmp);
-  osip_list_add (sip->vias, via, 0);
-  osip_free (tmp);
-
+  sprintf (tmp, "z9hG4bK%u", number);
+  br->gvalue = osip_strdup(tmp);
   return 0;
 }
