@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -55,6 +55,9 @@ int eXosip_register_remove (int rid)
       return -1;
     }
   jr->r_reg_period = 0;
+  REMOVE_ELEMENT (eXosip.j_reg, jr);
+  eXosip_reg_free (jr);
+  jr=NULL;
   return 0;
 }
 
@@ -62,7 +65,6 @@ static int
 _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
 {
   osip_message_t *reg = NULL;
-  struct eXosip_net *net;
   int i;
 
   reg = NULL;
@@ -103,76 +105,9 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
             int length = strlen (reg->cseq->number);
 
 
-            osip_authorization_t *aut;
-            osip_proxy_authorization_t *proxy_aut;
+	    osip_list_special_free(&reg->authorizations, (void *(*)(void *)) &osip_authorization_free);
+	    osip_list_special_free(&reg->proxy_authorizations, (void *(*)(void *)) &osip_proxy_authorization_free);
 
-            aut = (osip_authorization_t *) osip_list_get (reg->authorizations, 0);
-            while (aut != NULL)
-              {
-                osip_list_remove (reg->authorizations, 0);
-                osip_authorization_free (aut);
-                aut =
-                  (osip_authorization_t *) osip_list_get (reg->authorizations, 0);
-              }
-
-            proxy_aut =
-              (osip_proxy_authorization_t *) osip_list_get (reg->
-                                                            proxy_authorizations,
-                                                            0);
-            while (proxy_aut != NULL)
-              {
-                osip_list_remove (reg->proxy_authorizations, 0);
-                osip_proxy_authorization_free (proxy_aut);
-                proxy_aut =
-                  (osip_proxy_authorization_t *) osip_list_get (reg->
-                                                                proxy_authorizations,
-                                                                0);
-              }
-
-            if (0 == osip_strcasecmp (jr->transport, "udp"))
-              net = &eXosip.net_interfaces[0];
-            else if (0 == osip_strcasecmp (jr->transport, "tcp"))
-              net = &eXosip.net_interfaces[1];
-            else
-              {
-                OSIP_TRACE (osip_trace
-                            (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                             "eXosip: unsupported protocol '%s' (default to UDP)\n",
-                             jr->transport));
-                net = &eXosip.net_interfaces[0];
-              }
-
-            /*
-               modify the port number when masquerading is ON.
-             */
-            if (net->net_firewall_ip[0] != '\0')
-              {
-                int pos = 0;
-
-                while (!osip_list_eol (reg->contacts, pos))
-                  {
-                    osip_contact_t *co;
-
-                    co = (osip_contact_t *) osip_list_get (reg->contacts, pos);
-                    pos++;
-                    if (co != NULL && co->url != NULL && co->url->host != NULL
-                        && 0 == osip_strcasecmp (co->url->host,
-                                                 net->net_firewall_ip))
-                      {
-                        if (co->url->port == NULL &&
-                            0 != osip_strcasecmp (net->net_port, "5060"))
-                          {
-                            co->url->port = osip_strdup (net->net_port);
-                        } else if (co->url->port != NULL &&
-                                   0 != osip_strcasecmp (net->net_port,
-                                                         co->url->port))
-                          {
-                            osip_free (co->url->port);
-                            co->url->port = osip_strdup (net->net_port);
-                          }
-                      }
-                  }
-              }
 
             if (-1 == eXosip_update_top_via (reg))
               {
@@ -187,13 +122,17 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
             reg->cseq->number = (char *) osip_malloc (length + 2);      /* +2 like for 9 to 10 */
             sprintf (reg->cseq->number, "%i", osip_cseq_num);
 
-            {
-              osip_header_t *exp;
-
+	    {
+	      osip_header_t *exp;
+	      
               osip_message_header_get_byname (reg, "expires", 0, &exp);
-              osip_free (exp->hvalue);
-              exp->hvalue = (char *) osip_malloc (10);
-              snprintf (exp->hvalue, 9, "%i", jr->r_reg_period);
+              if (exp!=NULL)
+		{
+		  if (exp->hvalue!=NULL)
+		    osip_free (exp->hvalue);
+		  exp->hvalue = (char *) osip_malloc (10);
+		  snprintf (exp->hvalue, 9, "%i", jr->r_reg_period);
+		}
             }
 
             osip_message_force_update (reg);
@@ -205,13 +144,15 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
                 {
                   eXosip_add_authentication_information (reg, last_response);
                 }
+			  else
+                  eXosip_add_authentication_information (reg, NULL);
               osip_message_free (last_response);
             }
         }
     }
   if (reg == NULL)
     {
-      i = generating_register (&reg, jr->transport,
+      i = generating_register (jr, &reg, eXosip.transport,
                                jr->r_aor, jr->r_registrar, jr->r_contact,
                                jr->r_reg_period);
       if (i != 0)
@@ -234,16 +175,14 @@ eXosip_register_build_initial_register (const char *from, const char *proxy,
 
   *reg = NULL;
 
-  if (eXosip.net_interfaces[0].net_socket <= 0
-      && eXosip.net_interfaces[1].net_socket <= 0
-      && eXosip.net_interfaces[2].net_socket <= 0)
-    return -1;
-
   /* Avoid adding the same registration info twice to prevent mem leaks */
   for (jr = eXosip.j_reg; jr != NULL; jr = jr->next)
     {
       if (strcmp (jr->r_aor, from) == 0 && strcmp (jr->r_registrar, proxy) == 0)
         {
+	  REMOVE_ELEMENT (eXosip.j_reg, jr);
+	  eXosip_reg_free (jr);
+	  jr=NULL;
           break;
         }
     }
@@ -261,16 +200,12 @@ eXosip_register_build_initial_register (const char *from, const char *proxy,
       ADD_ELEMENT (eXosip.j_reg, jr);
     }
 
-  /* Guess transport from existing connections */
-  if (eXosip.net_interfaces[0].net_socket > 0)
-    osip_strncpy (jr->transport, "UDP", sizeof (jr->transport) - 1);
-  else if (eXosip.net_interfaces[1].net_socket > 0)
-    osip_strncpy (jr->transport, "TCP", sizeof (jr->transport) - 1);
-  else if (eXosip.net_interfaces[2].net_socket > 0)
-    osip_strncpy (jr->transport, "TLS", sizeof (jr->transport) - 1);
-
   /* build register */
   jr->r_reg_period = expires;
+  if (jr->r_reg_period <= 0)      /* too low */
+    jr->r_reg_period = 0;
+  else if (jr->r_reg_period < 100)      /* too low */
+    jr->r_reg_period = 100;
 
   i = _eXosip_register_build_register (jr, reg);
   if (i != 0)
@@ -281,6 +216,7 @@ eXosip_register_build_initial_register (const char *from, const char *proxy,
       *reg = NULL;
       return i;
     }
+
   return jr->r_id;
 }
 
@@ -303,8 +239,8 @@ eXosip_register_build_register (int rid, int expires, osip_message_t ** reg)
     } /* unregistration */
   else if (jr->r_reg_period > 3600)
     jr->r_reg_period = 3600;
-  else if (jr->r_reg_period < 200)      /* too low */
-    jr->r_reg_period = 200;
+  else if (jr->r_reg_period < 100)      /* too low */
+    jr->r_reg_period = 100;
 
   if (jr->r_last_tr != NULL)
     {
@@ -334,7 +270,6 @@ eXosip_register_send_register (int rid, osip_message_t * reg)
   osip_transaction_t *transaction;
   osip_event_t *sipevent;
   eXosip_reg_t *jr;
-  char *transport;
   int i;
 
   jr = eXosip_reg_find (rid);
@@ -366,10 +301,7 @@ eXosip_register_send_register (int rid, osip_message_t * reg)
         }
     }
 
-  transport = _eXosip_transport_protocol (reg);
-  osip_strncpy (jr->transport, transport, sizeof (jr->transport) - 1);
-
-  i = osip_transaction_init (&transaction, NICT, eXosip.j_osip, reg);
+  i = _eXosip_transaction_init (&transaction, NICT, eXosip.j_osip, reg);
   if (i != 0)
     {
       /* TODO: release the j_call.. */
