@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ typedef char HASHHEX[HASHHEXLEN + 1];
 extern eXosip_t eXosip;
 
 /* Private functions */
-static void CvtHex (IN HASH Bin, OUT HASHHEX Hex);
+void CvtHex (IN HASH Bin, OUT HASHHEX Hex);
 static void DigestCalcHA1 (IN const char *pszAlg, IN const char *pszUserName,
                            IN const char *pszRealm,
                            IN const char *pszPassword,
@@ -59,7 +59,7 @@ static void DigestCalcResponse (IN HASHHEX HA1, IN const char *pszNonce,
                                 IN const char *pszDigestUri,
                                 IN HASHHEX HEntity, OUT HASHHEX Response);
 
-static void
+void
 CvtHex (IN HASH Bin, OUT HASHHEX Hex)
 {
   unsigned short i;
@@ -193,19 +193,18 @@ end:
 
 
 int
-__eXosip_create_authorization_header (osip_message_t * previous_answer,
+__eXosip_create_authorization_header (osip_www_authenticate_t *wa,
                                       const char *rquri, const char *username,
                                       const char *passwd, const char *ha1,
                                       osip_authorization_t ** auth,
-                                      const char *method)
+                                      const char *method,
+									  const char *pCNonce,
+									  int iNonceCount)
 {
   osip_authorization_t *aut;
-  osip_www_authenticate_t *wa = NULL;
 
   char *qop=NULL;
 
-  osip_message_get_www_authenticate (previous_answer, 0, &wa);
-
   /* make some test */
   if (passwd == NULL)
     return -1;
@@ -290,24 +289,24 @@ __eXosip_create_authorization_header (osip_message_t * previous_answer,
     HASHHEX Response;
     const char *pha1 = NULL;
 
-    if (qop!=NULL)
-      {
 	if (qop!=NULL)
-	  {
+      {
 	    /* only accept qop="auth" */
 	    pszQop = osip_strdup("auth");
-	  }
-	szNonceCount = osip_strdup ("00000001");
-	pszCNonce = osip_strdup ("0a4f113b");
+
+		szNonceCount = osip_malloc(10);
+		snprintf(szNonceCount, 9, "%.8i", iNonceCount);
+
+		pszCNonce = osip_strdup (pCNonce);
 	
-	osip_authorization_set_message_qop (aut, osip_strdup ("auth"));
-	osip_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
+		osip_authorization_set_message_qop (aut, osip_strdup ("auth"));
+		osip_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
 	
-	{
-	  char *tmp = osip_malloc (strlen (pszCNonce) + 3);
-	  sprintf (tmp, "\"%s\"", pszCNonce);
-	  osip_authorization_set_cnonce (aut, tmp);
-	}
+		{
+		  char *tmp = osip_malloc (strlen (pszCNonce) + 3);
+		  sprintf (tmp, "\"%s\"", pszCNonce);
+		  osip_authorization_set_cnonce (aut, tmp);
+		}
       }
 
     pszPass = passwd;
@@ -348,18 +347,19 @@ __eXosip_create_authorization_header (osip_message_t * previous_answer,
 }
 
 int
-__eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
-                                            const char *rquri,
+__eXosip_create_proxy_authorization_header (osip_proxy_authenticate_t *wa,
+											const char *rquri,
                                             const char *username,
                                             const char *passwd,
                                             const char *ha1,
                                             osip_proxy_authorization_t **
-                                            auth, const char *method)
+                                            auth, const char *method,
+											const char *pCNonce,
+											int iNonceCount)
 {
   osip_proxy_authorization_t *aut;
-  osip_proxy_authenticate_t *wa;
 
-  osip_message_get_proxy_authenticate (previous_answer, 0, &wa);
+  char *qop=NULL;
 
   /* make some test */
   if (passwd == NULL)
@@ -421,6 +421,10 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
   }
   osip_proxy_authorization_set_algorithm (aut, osip_strdup ("MD5"));
 
+  qop = osip_www_authenticate_get_qop_options (wa);
+  if (qop==NULL || qop[0]=='\0' || strlen(qop)<4)
+    qop=NULL;
+
   {
     char *pszNonce = NULL;
     char *pszCNonce = NULL;
@@ -445,18 +449,26 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
       return -1;
     pszNonce = osip_strdup_without_quote (osip_www_authenticate_get_nonce (wa));
 
-    /* should upgrade szNonceCount */
-    /* should add szNonceCount in aut */
-    /* should upgrade pszCNonce */
-    /* should add pszCNonce in aut */
-
-    if (osip_proxy_authenticate_get_qop_options (wa) != NULL)
+    if (qop!=NULL)
       {
-        szNonceCount = osip_strdup ("00000001");
-        /* MUST be incremented on each */
-        pszQop = osip_strdup (osip_proxy_authenticate_get_qop_options (wa));
-        pszCNonce = osip_strdup ("234abcc436e2667097e7fe6eia53e8dd");
+	    /* only accept qop="auth" */
+	    pszQop = osip_strdup("auth");
+		
+		szNonceCount = osip_malloc(10);
+		snprintf(szNonceCount, 9, "%.8i", iNonceCount);
+
+		pszCNonce = osip_strdup (pCNonce);
+
+		osip_proxy_authorization_set_message_qop (aut, osip_strdup ("auth"));
+		osip_proxy_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
+
+		{
+		  char *tmp = osip_malloc (strlen (pszCNonce) + 3);
+		  sprintf (tmp, "\"%s\"", pszCNonce);
+		  osip_proxy_authorization_set_cnonce (aut, tmp);
+		}
       }
+
     if (ha1 && ha1[0])
       {
         /* Depending on algorithm=md5 */
@@ -489,3 +501,72 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
   *auth = aut;
   return 0;
 }
+
+int _eXosip_store_nonce(const char *call_id, osip_proxy_authenticate_t *wa)
+{
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, call_id)==0
+			&& osip_strcasecmp(http_auth->wa->realm, wa->realm)==0)
+		{
+			osip_proxy_authenticate_free(http_auth->wa);
+			http_auth->wa=NULL;
+			osip_proxy_authenticate_clone(wa, &(http_auth->wa));
+			http_auth->iNonceCount = 1;
+			if (http_auth->wa==NULL)
+				memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+
+	/* not found */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+		{
+			snprintf(http_auth->pszCallId, sizeof(http_auth->pszCallId),
+				call_id);
+			snprintf(http_auth->pszCNonce, sizeof(http_auth->pszCNonce),
+				"0a4f113b");
+			http_auth->iNonceCount = 1;
+			osip_proxy_authenticate_clone(wa, &(http_auth->wa));
+			if (http_auth->wa==NULL)
+				memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+
+	OSIP_TRACE (osip_trace
+                (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                 "Compile with higher MAX_EXOSIP_HTTP_AUTH value (current=%i)\n", MAX_EXOSIP_HTTP_AUTH));
+	return -1;
+}
+
+int _eXosip_delete_nonce(const char *call_id)
+{
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, call_id)==0)
+		{
+			osip_proxy_authenticate_free(http_auth->wa);
+			memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+	return -1;
+}
