@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -90,6 +90,8 @@ eXosip_event_init_for_call (int type, eXosip_call_t * jc,
   return je;
 }
 
+#ifndef MINISIZE
+
 eXosip_event_t *
 eXosip_event_init_for_subscribe (int type, eXosip_subscribe_t * js,
                                  eXosip_dialog_t * jd, osip_transaction_t * tr)
@@ -146,6 +148,8 @@ eXosip_event_init_for_notify (int type, eXosip_notify_t * jn,
   return je;
 }
 
+#endif
+
 eXosip_event_t *
 eXosip_event_init_for_reg (int type, eXosip_reg_t * jr, osip_transaction_t * tr)
 {
@@ -189,6 +193,7 @@ eXosip_event_init (eXosip_event_t ** je, int type)
   memset (*je, 0, sizeof (eXosip_event_t));
   (*je)->type = type;
 
+#if !defined(_WIN32_WCE)
   if (type == EXOSIP_CALL_NOANSWER)
     {
       sprintf ((*je)->textinfo, "No answer for this Call!");
@@ -319,6 +324,7 @@ eXosip_event_init (eXosip_event_t ** je, int type)
     {
       (*je)->textinfo[0] = '\0';
     }
+#endif
   return 0;
 }
 
@@ -355,28 +361,17 @@ report_call_event (int evt, eXosip_call_t * jc,
   report_event (je, NULL);
 }
 
-void
-report_options_event (int evt, osip_transaction_t * tr)
-{
-  eXosip_event_t *je;
-
-  eXosip_event_init (&je, evt);
-  if (je == NULL)
-    return;
-
-  if (tr != NULL)
-    je->tid = tr->transactionid;
-
-  _eXosip_event_fill_messages (je, tr);
-  report_event (je, NULL);
-}
-
 int
 eXosip_event_add (eXosip_event_t * je)
 {
   int i = osip_fifo_add (eXosip.j_events, (void *) je);
 
+#ifdef OSIP_MT
+#if !defined (_WIN32_WCE)
   osip_cond_signal ((struct osip_cond *) eXosip.j_cond);
+#endif
+#endif
+
   __eXosip_wakeup_event ();
   return i;
 }
@@ -416,6 +411,24 @@ __eXosip_clock_gettime (unsigned int clock_id, struct timespec *time)
 #endif
 #endif
 
+#ifndef OSIP_MT
+
+eXosip_event_t *
+eXosip_event_wait (int tv_s, int tv_ms)
+{
+  eXosip_event_t *je = NULL;
+  je = (eXosip_event_t *) osip_fifo_tryget (eXosip.j_events);
+  if (je != NULL)
+    return je;
+  
+  eXosip_lock();
+  eXosip_retransmit_lost200ok();
+  eXosip_unlock();
+  return NULL;
+}
+
+#else
+
 eXosip_event_t *
 eXosip_event_wait (int tv_s, int tv_ms)
 {
@@ -478,6 +491,10 @@ eXosip_event_wait (int tv_s, int tv_ms)
     if (je != NULL)
       return je;
 
+	eXosip_lock();
+	eXosip_retransmit_lost200ok();
+	eXosip_unlock();
+
     if (tv_s == 0 && tv_ms == 0)
       return NULL;
 
@@ -509,6 +526,11 @@ eXosip_event_get ()
 {
   eXosip_event_t *je;
 
+  char buf[500];
+  jpipe_read (eXosip.j_socketctl_event, buf, 499);
+
   je = (eXosip_event_t *) osip_fifo_get (eXosip.j_events);
   return je;
 }
+
+#endif
