@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@
 
 extern eXosip_t eXosip;
 
+extern int ipv6_enable;
+
 /* Private functions */
 static int dialog_fill_route_set (osip_dialog_t * dialog,
                                   osip_message_t * request);
@@ -82,6 +84,186 @@ via_branch_new_random (void)
   return osip_build_random_number ();
 }
 
+int
+_eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer)
+{
+  osip_via_t *via;
+  osip_from_t *a_from;
+  char *contact=NULL;
+  char locip[65];
+  char firewall_ip[65];
+  char firewall_port[10];
+  int len;
+
+  if (eXosip.eXtl==NULL)
+    return -1;
+  if (request==NULL)
+    return -1;
+
+  if (eXosip.eXtl->tl_get_masquerade_contact!=NULL)
+    {
+      eXosip.eXtl->tl_get_masquerade_contact(firewall_ip, sizeof(firewall_ip),
+					     firewall_port, sizeof(firewall_port));
+    }
+
+  /* search for topmost Via which indicate the transport protocol */
+  via = (osip_via_t *) osip_list_get (&request->vias, 0);
+  if (via == NULL || via->protocol == NULL)
+    {
+      OSIP_TRACE (osip_trace
+                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                   "eXosip: missing via header\n"));
+      return -1;
+    }
+
+  if (answer==NULL)
+    a_from = request->from;
+  else
+    a_from = answer->to;
+
+  if (a_from == NULL || a_from->url == NULL)
+    return -1;
+
+  /*guess the local ip since req uri is known */
+  memset(locip, '\0', sizeof(locip));
+
+  if (a_from->url->username != NULL)
+    len = 2 + 4 + strlen (a_from->url->username) + 1 + 100 + 6 + 10 + strlen(eXosip.transport);
+  else
+    len = 2 + 4 + 100 + 6 + 10 + strlen(eXosip.transport);
+
+  contact = (char *) osip_malloc (len+1);
+  if (firewall_ip[0] != '\0')
+    {
+      char *c_address = request->req_uri->host;
+
+      struct addrinfo *addrinfo;
+      struct __eXosip_sockaddr addr;
+      int i;
+
+      i = eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_TCP);
+      if (i == 0)
+        {
+	  memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
+	  eXosip_freeaddrinfo (addrinfo);
+	  c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
+	  OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
+				  "eXosip: here is the resolved destination host=%s\n",
+				  c_address));
+        }
+
+      if (eXosip_is_public_address (c_address))
+        {
+	  memcpy(locip, firewall_ip, sizeof(locip));
+	}
+    }
+
+  if (locip[0] == '\0')
+    {
+      eXosip_guess_ip_for_via (eXosip.eXtl->proto_family, locip, 49);
+      if (locip[0]=='\0')
+	{
+	  OSIP_TRACE (osip_trace
+		      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		       "eXosip: no default interface defined\n"));
+	  return -1;
+	}
+    }
+
+  if (eXosip.eXtl->proto_family == AF_INET6)
+    {
+      if (a_from->url->username != NULL)
+	snprintf (contact, len, "<sip:%s@[%s]:%s>", a_from->url->username,
+		  locip, firewall_port);
+      else
+	snprintf (contact, len-strlen(eXosip.transport)-10, "<sip:[%s]:%s>", locip, firewall_port);
+    }
+  else
+    {
+      if (a_from->url->username != NULL)
+	snprintf (contact, len, "<sip:%s@%s:%s>", a_from->url->username,
+		  locip, firewall_port);
+      else
+	snprintf (contact, len-strlen(eXosip.transport)-10, "<sip:%s:%s>", locip, firewall_port);
+    }
+  if (osip_strcasecmp(eXosip.transport, "UDP")!=0)
+    {
+      contact[strlen(contact)-1]='\0';
+      strcat(contact, ";transport=");
+      strcat(contact, eXosip.transport);
+      strcat(contact, ">");
+    }
+  osip_message_set_contact (request, contact);
+  osip_free (contact);
+
+  return 0;
+}
+
+int
+_eXosip_request_add_via(osip_message_t *request, const char *transport, const char *locip)
+{
+  char tmp[200];
+  const char *ip=NULL;
+  char firewall_ip[65];
+  char firewall_port[10];
+
+  if (request==NULL)
+    return -1;
+
+  if (request->call_id==NULL)
+    return -1;
+
+  if (locip==NULL && request->call_id->host==NULL)
+    return -1;
+
+  if (locip!=NULL)
+    ip = locip;
+  else if (request->call_id->host!=NULL)
+    ip = request->call_id->host;
+
+  firewall_ip[0] = '\0';
+  firewall_port[0] = '\0';
+  if (eXosip.eXtl!=NULL && eXosip.eXtl->tl_get_masquerade_contact!=NULL)
+    {
+      eXosip.eXtl->tl_get_masquerade_contact(firewall_ip, sizeof(firewall_ip),
+					     firewall_port, sizeof(firewall_port));
+    }
+
+#ifdef MASQUERADE_VIA
+  /* this helps to work with a server that don't handle the
+     "received" parameter correctly. Some still exists. */
+  if (firewall_ip[0] != '\0')
+    {
+      ip = firewall_ip;
+    }
+#endif
+
+  if (firewall_port[0] == '\0')
+    {
+      snprintf(firewall_port, sizeof(firewall_port), "5060");
+    }
+
+  if (eXosip.eXtl->proto_family == AF_INET6)
+    snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u",
+	      eXosip.transport,
+	      ip, firewall_port, via_branch_new_random ());
+  else
+    {
+      if (eXosip.use_rport!=0)
+	snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
+		  eXosip.transport,
+		  ip, firewall_port, via_branch_new_random ());
+      else
+	snprintf (tmp, 200, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u",
+		  eXosip.transport,
+		  ip, firewall_port, via_branch_new_random ());	  
+    }
+  
+  osip_message_set_via (request, tmp);
+  
+  return 0;
+}
+
 /* prepare a minimal request (outside of a dialog) with required headers */
 /* 
    method is the type of request. ("INVITE", "REGISTER"...)
@@ -99,25 +281,26 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
    */
   int i;
   osip_message_t *request;
-  char locip[50];
+  char locip[65];
   int doing_register;
   char *register_callid_number = NULL;
-  struct eXosip_net *net;
 
-  if (0 == osip_strcasecmp (transport, "udp"))
-    net = &eXosip.net_interfaces[0];
-  else if (0 == osip_strcasecmp (transport, "tcp"))
-    net = &eXosip.net_interfaces[1];
-  else
+  *dest = NULL;
+
+  if (eXosip.eXtl==NULL)
+    return -1;
+
+  /*guess the local ip since req uri is known */
+  memset(locip, '\0', sizeof(locip));
+  eXosip_guess_ip_for_via (eXosip.eXtl->proto_family, locip, 49);
+  if (locip[0]=='\0')
     {
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (default to UDP)\n"));
-      net = &eXosip.net_interfaces[0];
+                   "eXosip: no default interface defined\n"));
+      return -1;
     }
 
-  *dest = NULL;
-
   i = osip_message_init (&request);
   if (i != 0)
     return -1;
@@ -151,6 +334,44 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
                        to));
           goto brood_error_1;
         }
+
+	  /* REMOVE ALL URL PARAMETERS from to->url headers and add them as headers */
+	  if (request->to!=NULL &&
+		  request->to->url!=NULL)
+	  {
+		  osip_uri_t *url = request->to->url;
+		  while (osip_list_size(&url->url_headers)>0)
+		  {
+			  osip_uri_header_t *u_header;
+			  u_header = (osip_uri_param_t *) osip_list_get (&url->url_headers, 0);
+			  if (u_header==NULL)
+				break;
+
+			  if (osip_strcasecmp(u_header->gname, "from")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "to")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "call-id")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "cseq")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "via")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "contact")==0)
+			  {
+			  }
+			  else
+				  osip_message_set_header(request, u_header->gname, u_header->gvalue);
+			  osip_list_remove (&url->url_headers, 0);
+			  osip_uri_param_free (u_header);
+		  }
+	  }
+
       if (proxy != NULL && proxy[0] != 0)
         {                       /* equal to a pre-existing route set */
           /* if the pre-existing route set contains a "lr" (compliance
@@ -173,7 +394,7 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
               osip_uri_clone (request->to->url, &(request->req_uri));
               /* "[request] MUST includes a Route header field containing
                  the route set values in order." */
-              osip_list_add (request->routes, o_proxy, 0);
+              osip_list_add (&request->routes, o_proxy, 0);
           } else
             /* if the first URI of route set does not contain "lr", the req_uri
                is set to the first uri of route set */
@@ -197,24 +418,15 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
             goto brood_error_1;
         }
     }
-  /*guess the local ip since req uri is known */
-#ifdef SM
-  eXosip_get_localip_for (request->req_uri->host, locip, 49);
-#else
-  if (0 == osip_strcasecmp (transport, "udp"))
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-  else if (0 == osip_strcasecmp (transport, "tcp"))
-    eXosip_guess_ip_for_via (eXosip.net_interfaces[1].net_ip_family, locip, 49);
-  else
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol (default to UDP)\n"));
-      eXosip_guess_ip_for_via (eXosip.net_interfaces[0].net_ip_family, locip, 49);
-    }
-#endif
+
+  if (locip[0]=='\0')
+    goto brood_error_1;
+
   /* set To and From */
   osip_message_set_from (request, from);
+  if (request->from==NULL)
+    goto brood_error_1;
+
   /* add a tag */
   osip_from_set_tag (request->from, osip_from_tag_new_random ());
 
@@ -234,7 +446,9 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
     if (doing_register)
       register_callid_number = cidrand;
 
+#if 0
     osip_call_id_set_host (callid, osip_strdup (locip));
+#endif
     request->call_id = callid;
 
     i = osip_cseq_init (&cseq);
@@ -246,169 +460,13 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
     request->cseq = cseq;
   }
 
+  i = _eXosip_request_add_via(request, transport, locip);
+  if (i != 0)
+    goto brood_error_1;
+
   /* always add the Max-Forward header */
   osip_message_set_max_forwards (request, "70"); /* a UA should start a request with 70 */
 
-#define MASQUERADE_VIA
-#ifdef MASQUERADE_VIA
-  /* should be useless with compliant UA */
-  if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-    {
-      char *c_address = request->req_uri->host;
-
-      struct addrinfo *addrinfo;
-      struct __eXosip_sockaddr addr;
-
-      i =
-        eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_UDP);
-      if (i == 0)
-        {
-          memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-          freeaddrinfo (addrinfo);
-          c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                       "eXosip: here is the resolved destination host=%s\n",
-                       c_address));
-        }
-
-      if (eXosip_is_public_address (c_address))
-        {
-          char tmp[200];
-
-          snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                    transport, eXosip.net_interfaces[0].net_firewall_ip,
-                    net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-      } else
-        {
-          char tmp[200];
-
-          if (net->net_ip_family == AF_INET6)
-            snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          else
-            snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-        }
-  } else
-    {
-      char tmp[200];
-
-      if (net->net_ip_family == AF_INET6)
-        snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                  locip, net->net_port, via_branch_new_random ());
-      else
-        snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                  transport, locip, net->net_port, via_branch_new_random ());
-      osip_message_set_via (request, tmp);
-    }
-
-#else
-  {
-    char tmp[200];
-
-    if (net->net_ip_family == AF_INET6)
-      spnrintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                locip, net->net_port, via_branch_new_random ());
-    else
-      spnrintf (tmp, 200, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u", transport,
-                locip, net->net_port, via_branch_new_random ());
-
-    osip_message_set_via (request, tmp);
-  }
-#endif
-
-  /* add specific headers for each kind of request... */
-
-  if (0 == strcmp ("INVITE", method) || 0 == strcmp ("SUBSCRIBE", method))
-    {
-      char *contact;
-      osip_from_t *a_from;
-      int i;
-
-      i = osip_from_init (&a_from);
-      if (i == 0)
-        i = osip_from_parse (a_from, from);
-
-      if (i == 0 && a_from != NULL
-          && a_from->url != NULL)
-        {
-	  if (a_from->url->username != NULL)
-	    contact = (char *) osip_malloc (50 + strlen (a_from->url->username));
-	  else
-	    contact = (char *) osip_malloc (50);
-          if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-            {
-              char *c_address = request->req_uri->host;
-
-              struct addrinfo *addrinfo;
-              struct __eXosip_sockaddr addr;
-
-              i =
-                eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060,
-                                     IPPROTO_TCP);
-              if (i == 0)
-                {
-                  memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-                  freeaddrinfo (addrinfo);
-                  c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-                  OSIP_TRACE (osip_trace
-                              (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                               "eXosip: here is the resolved destination host=%s\n",
-                               c_address));
-                }
-
-              if (eXosip_is_public_address (c_address))
-                {
-		  if (a_from->url->username != NULL)
-		    sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-			     eXosip.net_interfaces[0].net_firewall_ip,
-			     net->net_port);
-		  else
-		    sprintf (contact, "<sip:%s:%s>",
-			     eXosip.net_interfaces[0].net_firewall_ip,
-			     net->net_port);
-		}
-	      else
-		{
-		  if (a_from->url->username != NULL)
-		    sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-			     locip, net->net_port);
-		  else
-		    sprintf (contact, "<sip:%s:%s>",
-			     locip, net->net_port);
-                }
-	    }
-	  else
-	    {
-	      if (a_from->url->username != NULL)
-		sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-			 locip, net->net_port);
-	      else
-		sprintf (contact, "<sip:%s:%s>",
-			 locip, net->net_port);
-            }
-          osip_message_set_contact (request, contact);
-          osip_free (contact);
-        }
-      osip_from_free (a_from);
-
-      /* This is probably useless for other messages */
-      osip_message_set_allow (request, "INVITE");
-      osip_message_set_allow (request, "ACK");
-      osip_message_set_allow (request, "UPDATE");
-      osip_message_set_allow (request, "INFO");
-      osip_message_set_allow (request, "CANCEL");
-      osip_message_set_allow (request, "BYE");
-      osip_message_set_allow (request, "OPTIONS");
-      osip_message_set_allow (request, "REFER");
-      osip_message_set_allow (request, "SUBSCRIBE");
-      osip_message_set_allow (request, "NOTIFY");
-      osip_message_set_allow (request, "MESSAGE");
-    }
-
   if (0 == strcmp ("REGISTER", method))
     {
   } else if (0 == strcmp ("INFO", method))
@@ -430,13 +488,22 @@ brood_error_1:
 }
 
 int
-generating_register (osip_message_t ** reg, char *transport, char *from,
+generating_register (eXosip_reg_t * jreg, osip_message_t ** reg, char *transport, char *from,
                      char *proxy, char *contact, int expires)
 {
   osip_from_t *a_from;
   int i;
-  char locip[50];
-  struct eXosip_net *net;
+  char locip[65];
+  char firewall_ip[65];
+  char firewall_port[10];
+  if (eXosip.eXtl==NULL)
+    return -1;
+
+  if (eXosip.eXtl->tl_get_masquerade_contact!=NULL)
+    {
+      eXosip.eXtl->tl_get_masquerade_contact(firewall_ip, sizeof(firewall_ip),
+					     firewall_port, sizeof(firewall_port));
+    }
 
   i =
     generating_request_out_of_dialog (reg, "REGISTER", NULL, transport, from,
@@ -444,26 +511,19 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
   if (i != 0)
     return -1;
 
-  i = _eXosip_find_protocol (*reg);
-  if (i == IPPROTO_UDP)
-    net = &eXosip.net_interfaces[0];
-  else if (i == IPPROTO_TCP)
-    net = &eXosip.net_interfaces[1];
-  else
+  memset(locip, '\0', sizeof(locip));
+  eXosip_guess_ip_for_via (eXosip.eXtl->proto_family, locip, 49);
+
+  if (locip[0]=='\0')
     {
-      net = &eXosip.net_interfaces[0];
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol\n"));
+                   "eXosip: no default interface defined\n"));
+      osip_message_free(*reg);
+      *reg=NULL;
       return -1;
     }
 
-#ifdef SM
-  eXosip_get_localip_for ((*reg)->req_uri->host, locip, 49);
-#else
-  eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#endif
-
   if (contact == NULL)
     {
       i = osip_from_init (&a_from);
@@ -473,8 +533,17 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
       if (i == 0 && a_from != NULL
           && a_from->url != NULL && a_from->url->username != NULL)
         {
-          contact = (char *) osip_malloc (50 + strlen (a_from->url->username));
-          if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
+	  int len = 50 + strlen (a_from->url->username);
+	  if (transport!=NULL && osip_strcasecmp(transport, "UDP")!=0)
+	    {
+	      len = len + strlen(transport) + 12; /* strlen(";transport=") */
+	    }
+	  if (jreg->r_line[0]!='\0')
+	    {
+	      len = len + strlen(jreg->r_line) + 12; /* strlen(";line=") */
+	    }
+	  contact = (char *) osip_malloc (len);
+          if (firewall_ip[0] != '\0')
             {
               char *c_address = (*reg)->req_uri->host;
 
@@ -487,7 +556,7 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
               if (i == 0)
                 {
                   memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-                  freeaddrinfo (addrinfo);
+                  eXosip_freeaddrinfo (addrinfo);
                   c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
                   OSIP_TRACE (osip_trace
                               (__FILE__, __LINE__, OSIP_INFO1, NULL,
@@ -498,18 +567,34 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
               if (eXosip_is_public_address (c_address))
                 {
                   sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                           eXosip.net_interfaces[0].net_firewall_ip,
-                           net->net_port);
+                           firewall_ip,
+                           firewall_port);
               } else
                 {
                   sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                           locip, net->net_port);
+                           locip, firewall_port);
                 }
           } else
             {
               sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
-                       locip, net->net_port);
+                       locip, firewall_port);
             }
+	  
+	  if (transport!=NULL && osip_strcasecmp(transport, "UDP")!=0)
+	    {
+	      contact[strlen(contact)-1]='\0';
+	      strcat(contact, ";transport=");
+	      strcat(contact, transport);
+	      strcat(contact, ">");
+	    }
+
+	  if (jreg->r_line[0]!='\0')
+	    {
+	      contact[strlen(contact)-1]='\0';
+	      strcat(contact, ";line=");
+	      strcat(contact, jreg->r_line);
+	      strcat(contact, ">");
+	    }
 
           osip_message_set_contact (*reg, contact);
           osip_free (contact);
@@ -532,6 +617,8 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
   return 0;
 }
 
+#ifndef MINISIZE
+
 int
 generating_publish (osip_message_t ** message, const char *to,
                     const char *from, const char *route)
@@ -554,6 +641,8 @@ generating_publish (osip_message_t ** message, const char *to,
   return 0;
 }
 
+#endif
+
 static int
 dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
 {
@@ -568,16 +657,7 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
 
   /* AMD bug: fixed 17/06/2002 */
 
-#ifdef OSIP_FUTURE_FIX_2_3
-  route = (osip_route_t *) osip_list_get (dialog->route_set, 0);
-#else
-  if (dialog->type == CALLER)
-    {
-      pos = osip_list_size (dialog->route_set) - 1;
-      route = (osip_route_t *) osip_list_get (dialog->route_set, pos);
-  } else
-    route = (osip_route_t *) osip_list_get (dialog->route_set, 0);
-#endif
+  route = (osip_route_t *) osip_list_get (&dialog->route_set, 0);
 
   osip_uri_uparam_get_byname (route->url, "lr", &lr_param);
   if (lr_param != NULL)         /* the remote target URI is the req_uri! */
@@ -589,23 +669,16 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
          the route set values in order." */
       /* AMD bug: fixed 17/06/2002 */
       pos = 0;                  /* first element is at index 0 */
-      while (!osip_list_eol (dialog->route_set, pos))
+      while (!osip_list_eol (&dialog->route_set, pos))
         {
           osip_route_t *route2;
 
-          route = osip_list_get (dialog->route_set, pos);
+          route = osip_list_get (&dialog->route_set, pos);
           i = osip_route_clone (route, &route2);
           if (i != 0)
             return -1;
-#ifdef OSIP_FUTURE_FIX_2_3
-          osip_list_add (request->routes, route2, -1);
-#else
-          if (dialog->type == CALLER)
-            osip_list_add (request->routes, route2, 0);
-          else
-            osip_list_add (request->routes, route2, -1);
+          osip_list_add (&request->routes, route2, -1);
           pos++;
-#endif
         }
       return 0;
     }
@@ -622,34 +695,18 @@ dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
      the remainder of the route set values in order. */
   pos = 0;                      /* yes it is */
 
-  while (!osip_list_eol (dialog->route_set, pos))       /* not the first one in the list */
+  while (!osip_list_eol (&dialog->route_set, pos))       /* not the first one in the list */
     {
       osip_route_t *route2;
 
-      route = osip_list_get (dialog->route_set, pos);
+      route = osip_list_get (&dialog->route_set, pos);
       i = osip_route_clone (route, &route2);
       if (i != 0)
         return -1;
-#ifdef OSIP_FUTURE_FIX_2_3
-      if (!osip_list_eol (dialog->route_set, pos + 1))
-        osip_list_add (request->routes, route2, -1);
+      if (!osip_list_eol (&dialog->route_set, pos + 1))
+        osip_list_add (&request->routes, route2, -1);
       else
         osip_route_free (route2);
-#else
-      if (dialog->type == CALLER)
-        {
-          if (pos != osip_list_size (dialog->route_set) - 1)
-            osip_list_add (request->routes, route2, 0);
-          else
-            osip_route_free (route2);
-      } else
-        {
-          if (!osip_list_eol (dialog->route_set, pos + 1))
-            osip_list_add (request->routes, route2, -1);
-          else
-            osip_route_free (route2);
-        }
-#endif
       pos++;
     }
 
@@ -676,8 +733,19 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
 {
   int i;
   osip_message_t *request;
-  char locip[50];
-  struct eXosip_net *net;
+  char locip[65];
+  char firewall_ip[65];
+  char firewall_port[10];
+
+  *dest=NULL;
+
+  if (eXosip.eXtl==NULL)
+    return -1;
+  if (eXosip.eXtl->tl_get_masquerade_contact!=NULL)
+    {
+      eXosip.eXtl->tl_get_masquerade_contact(firewall_ip, sizeof(firewall_ip),
+					     firewall_port, sizeof(firewall_port));
+    }
 
   i = osip_message_init (&request);
   if (i != 0)
@@ -692,23 +760,18 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
       return -1;
     }
 
-  if (0 == osip_strcasecmp (transport, "udp"))
-    net = &eXosip.net_interfaces[0];
-  else if (0 == osip_strcasecmp (transport, "tcp"))
-    net = &eXosip.net_interfaces[1];
-  else
+
+  memset(locip, '\0', sizeof(locip));
+  eXosip_guess_ip_for_via (eXosip.eXtl->proto_family, locip, 49);
+  if (locip[0]=='\0')
     {
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol -%s- (default to UDP)\n"));
-      net = &eXosip.net_interfaces[0];
+                   "eXosip: no default interface defined\n"));
+      osip_message_free(request);
+      return -1;
     }
 
-#ifdef SM
-  eXosip_get_localip_for (dialog->remote_contact_uri->url->host, locip, 49);
-#else
-  eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#endif
   /* prepare the request-line */
   request->sip_method = osip_strdup (method);
   request->sip_version = osip_strdup ("SIP/2.0");
@@ -716,7 +779,7 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
   request->reason_phrase = NULL;
 
   /* and the request uri???? */
-  if (osip_list_eol (dialog->route_set, 0))
+  if (osip_list_eol (&dialog->route_set, 0))
     {
       /* The UAC must put the remote target URI (to field) in the req_uri */
       i = osip_uri_clone (dialog->remote_contact_uri->url, &(request->req_uri));
@@ -772,86 +835,16 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
   osip_message_set_max_forwards (request, "70"); /* a UA should start a request with 70 */
 
 
-  /* even for ACK for 2xx (ACK within a dialog), the branch ID MUST
-     be a new ONE! */
-#ifdef MASQUERADE_VIA
-  /* should be useless with compliant UA */
-  if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
-    {
-      char *c_address = request->req_uri->host;
-
-      struct addrinfo *addrinfo;
-      struct __eXosip_sockaddr addr;
-
-      i =
-        eXosip_get_addrinfo (&addrinfo, request->req_uri->host, 5060, IPPROTO_UDP);
-      if (i == 0)
-        {
-          memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-          freeaddrinfo (addrinfo);
-          c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                       "eXosip: here is the resolved destination host=%s\n",
-                       c_address));
-        }
-
-      if (eXosip_is_public_address (c_address))
-        {
-          char tmp[200];
-
-          sprintf (tmp, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u", transport,
-                   eXosip.net_interfaces[0].net_firewall_ip,
-                   net->net_port, via_branch_new_random ());
-          osip_message_set_via (request, tmp);
-      } else
-        {
-          char tmp[200];
-
-          if (eXosip.net_interfaces[0].net_ip_family == AF_INET6)
-            snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-          else
-            snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                      transport, locip, net->net_port, via_branch_new_random ());
-
-          osip_message_set_via (request, tmp);
-        }
-  } else
-    {
-      char tmp[200];
-
-      if (net->net_ip_family == AF_INET6)
-        snprintf (tmp, 200, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-                  locip, net->net_port, via_branch_new_random ());
-      else
-        snprintf (tmp, 200, "SIP/2.0/%s %s:%s;rport;branch=z9hG4bK%u",
-                  transport, locip, net->net_port, via_branch_new_random ());
-
-      osip_message_set_via (request, tmp);
-    }
-
-#else
-  {
-    char tmp[200];
-
-    if (net->net_ip_family == AF_INET6)
-      sprintf (tmp, "SIP/2.0/%s [%s]:%s;branch=z9hG4bK%u", transport,
-               locip, net->net_port, via_branch_new_random ());
-    else
-      sprintf (tmp, "SIP/2.0/%s %s:%s;branch=z9hG4bK%u", transport,
-               locip, net->net_port, via_branch_new_random ());
-
-    osip_message_set_via (request, tmp);
-  }
-#endif
+  i = _eXosip_request_add_via(request, transport, locip);
+  if (i != 0)
+    goto grwd_error_1;
 
   /* add specific headers for each kind of request... */
 
   {
     char contact[200];
 
-    if (net->net_firewall_ip[0] != '\0')
+    if (firewall_ip[0] != '\0')
       {
         char *c_address = request->req_uri->host;
 
@@ -864,7 +857,7 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
         if (i == 0)
           {
             memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-            freeaddrinfo (addrinfo);
+            eXosip_freeaddrinfo (addrinfo);
             c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
             OSIP_TRACE (osip_trace
                         (__FILE__, __LINE__, OSIP_INFO1, NULL,
@@ -876,16 +869,16 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
           {
             sprintf (contact, "<sip:%s@%s:%s>",
                      dialog->local_uri->url->username,
-                     eXosip.net_interfaces[0].net_firewall_ip, net->net_port);
+                     firewall_ip, firewall_port);
         } else
           {
             sprintf (contact, "<sip:%s@%s:%s>",
-                     dialog->local_uri->url->username, locip, net->net_port);
+                     dialog->local_uri->url->username, locip, firewall_port);
           }
     } else
       {
         sprintf (contact, "<sip:%s@%s:%s>", dialog->local_uri->url->username,
-                 locip, net->net_port);
+                 locip, firewall_port);
       }
     osip_message_set_contact (request, contact);
     /* Here we'll add the supported header if it's needed! */
@@ -981,7 +974,7 @@ generating_cancel (osip_message_t ** dest, osip_message_t * request_cancelled)
     i = osip_via_clone (via, &via2);
     if (i != 0)
       goto gc_error_1;
-    osip_list_add (request->vias, via2, -1);
+    osip_list_add (&request->vias, via2, -1);
   }
 
   /* add the same route-set than in the previous request */
@@ -990,13 +983,13 @@ generating_cancel (osip_message_t ** dest, osip_message_t * request_cancelled)
     osip_route_t *route;
     osip_route_t *route2;
 
-    while (!osip_list_eol (request_cancelled->routes, pos))
+    while (!osip_list_eol (&request_cancelled->routes, pos))
       {
-        route = (osip_route_t *) osip_list_get (request_cancelled->routes, pos);
+        route = (osip_route_t *) osip_list_get (&request_cancelled->routes, pos);
         i = osip_route_clone (route, &route2);
         if (i != 0)
           goto gc_error_1;
-        osip_list_add (request->routes, route2, -1);
+        osip_list_add (&request->routes, route2, -1);
         pos++;
       }
   }
